/*                                                                            */

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/graphics.h>
	#include <proto/intuition.h>
	#include <proto/cybergraphics.h>
	#include <cybergraphx/cybergraphics.h>
	#include <proto/datatypes.h>
	#include <datatypes/pictureclass.h>
	#include <devices/timer.h>

	#include <string.h>
	#include <stdlib.h>

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

#ifdef __VBCC__
	struct Library *TimerBase;
#endif

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	struct List *read_dir (char *path)

	{
	struct List *list;
	if (list = AllocVec (sizeof(struct List),MEMF_CLEAR))
		{
		BPTR lock;
		NewList (list);
		if (lock = Lock (path,SHARED_LOCK))
			{
			struct FileInfoBlock *fib;
			if (fib = AllocDosObject (DOS_FIB,NULL))
				{
				if (Examine (lock,fib))
					{
					struct Node *node;
					int pathlen = strlen(path);
					int len;
					if (fib->fib_DirEntryType > 0)
						{
						while (ExNext (lock,fib))
							{
							if (fib->fib_DirEntryType < 0)
								{
								len = pathlen + strlen(fib->fib_FileName) + 2;
								if (node = AllocVec (sizeof(struct Node) + len,MEMF_CLEAR))
									{
									node->ln_Name = (STRPTR)(node + 1);
									strcpy (node->ln_Name,path);
									AddPart (node->ln_Name,fib->fib_FileName,len);
									AddTail (list,node);
									}
								}
							}
						}
					else
						{
						len = pathlen + strlen(fib->fib_FileName) + 2;
						if (node = AllocVec (sizeof(struct Node) + len,MEMF_CLEAR))
							{
							node->ln_Name = (STRPTR)(node + 1);
							strcpy (node->ln_Name,path);
							AddPart (node->ln_Name,fib->fib_FileName,len);
							AddTail (list,node);
							}
						}
					}
				FreeDosObject (DOS_FIB,fib);
				}
			UnLock (lock);
			}

		if (!(list->lh_Head->ln_Succ))
			{
			FreeVec (list);
			list = NULL;
			}
		}

	return (list);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void free_list (struct List *list)

	{
	struct Node *node;
	while (node = RemTail (list))
		FreeVec (node);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	BOOL load_image (char *name,UBYTE *array,long array_w,long array_h)

	{
	BOOL ok = FALSE;
	Object *o;

	if (o = NewDTObject (name,DTA_GroupID,GID_PICTURE,PDTA_Remap,FALSE,PDTA_DestMode,PMODE_V43,TAG_END))
		{
		struct BitMapHeader *bmhd;
		GetDTAttrs (o,PDTA_BitMapHeader,&bmhd,TAG_END);
		if (bmhd)
			{
			long w,h;
			w = array_w;
			h = bmhd->bmh_Height * array_w / bmhd->bmh_Width;
			if (h > array_h)
				{
				w = bmhd->bmh_Width * array_h / bmhd->bmh_Height;
				h = array_h;
				}

			DoMethod (o,PDTM_SCALE,w,h,0);
			DoMethod (o,PDTM_READPIXELARRAY,array + (array_h - h) / 2 * array_w * 3 + (array_w - w) / 2 * 3,PBPAFMT_RGB,array_w * 3,0,0,w,h);
			ok = TRUE;
			}
		DisposeDTObject (o);
		}

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void blend (UBYTE *op,UBYTE *np,UBYTE *ap,int w,int h,int alpha)

	{
	int negalpha = 255 - alpha;
	int i = w * h;

	while (i)
		{
		*ap++ = ((*op++) * negalpha + (*np++) * alpha) / 255;
		*ap++ = ((*op++) * negalpha + (*np++) * alpha) / 255;
		*ap++ = ((*op++) * negalpha + (*np++) * alpha) / 255;
		i --;
		}
	}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	struct timerequest *open_timer (long unit)

	{
	struct MsgPort *reply;
	struct timerequest *request;

	if (reply = CreateMsgPort ())
		{
		if (request = (struct timerequest *) CreateIORequest (reply,sizeof(struct timerequest)))
			{
			if (OpenDevice ("timer.device",unit,(struct IORequest *)request,0) == 0)
				{
				return (request);
				}

			DeleteIORequest ((struct IORequest *)request);
			}
		DeleteMsgPort (reply);
		}

	return (NULL);
	}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	void close_timer (struct timerequest *request)

	{
	struct MsgPort *reply;

	if (request)
		{
		reply = request->tr_node.io_Message.mn_ReplyPort;
		CloseDevice ((struct IORequest *)request);
		DeleteIORequest ((struct IORequest *)request);
		DeleteMsgPort (reply);
		}
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void start_timer (struct timerequest *request,ULONG secs,ULONG micro)

	{
	request->tr_node.io_Command = TR_ADDREQUEST;
	request->tr_time.tv_secs  = secs;
	request->tr_time.tv_micro = micro;
	SendIO ((struct IORequest *)request);
	}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	void stop_timer (struct timerequest *request)

	{
	if (!CheckIO ((struct IORequest *)request))
		AbortIO ((struct IORequest *)request);
	WaitIO  ((struct IORequest *)request);
	SetSignal (0,1L << request->tr_node.io_Message.mn_ReplyPort->mp_SigBit);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	struct RDArgs *rdargs;
	struct {
		char *path;
		long *w;
		long *h;
		} args = {"QNAS:Bilder/Wallpaper"};

	if (rdargs = ReadArgs ("PATH,W/N,H/N",(APTR)&args,NULL))
		{
		struct List *list;
		if (list = read_dir (args.path))
			{
			struct timerequest *timer;

			if (timer = open_timer (UNIT_VBLANK))
				{
				struct Window *win;

			#ifdef __VBCC__
				TimerBase = (struct Library *)timer->tr_node.io_Device;
			#endif

				if (win = OpenWindowTags (NULL,
						WA_Left,20,
						WA_Top,20,
						WA_InnerWidth,args.w ? *args.w : 320,
						WA_InnerHeight,args.h ? *args.h : 240,
						WA_Title,"Fade",
						WA_Flags,WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_ACTIVATE | WFLG_NOCAREREFRESH,
						WA_IDCMP,IDCMP_VANILLAKEY | IDCMP_CLOSEWINDOW,
						TAG_END))
					{
					UBYTE *fromarray = AllocVec (win->GZZWidth * win->GZZHeight * 3,MEMF_CLEAR);
					UBYTE *toarray = AllocVec (win->GZZWidth * win->GZZHeight * 3,MEMF_CLEAR);
					UBYTE *winarray = AllocVec (win->GZZWidth * win->GZZHeight * 3,MEMF_CLEAR);

					if (fromarray && toarray && winarray)
						{
						BOOL cont = TRUE;
						struct Node *node = list->lh_Head;
						struct timeval starttime;

						GetSysTime (&starttime);
						starttime.tv_secs -= 5;

						start_timer (timer,0,20000);

						do	{
							ULONG winsig = 1L << win->UserPort->mp_SigBit;
							ULONG timersig = 1L << timer->tr_node.io_Message.mn_ReplyPort->mp_SigBit;
							ULONG sigs = Wait (winsig | timersig | SIGBREAKF_CTRL_C);

							if (sigs & SIGBREAKF_CTRL_C)
								cont = FALSE;

							if (sigs & timersig)
								{
								struct timeval timediff;
								long alpha;

								WaitIO ((struct IORequest *)timer);
								start_timer (timer,0,20000);

								GetSysTime (&timediff);
								SubTime (&timediff,&starttime);
								alpha = (timediff.tv_secs * 50 + timediff.tv_micro / 20000) * 2;

								if (alpha >= 255)
									alpha = 255;

								blend (fromarray,toarray,winarray,win->GZZWidth,win->GZZHeight,alpha);
								WritePixelArray (winarray,0,0,win->GZZWidth * 3,win->RPort,win->BorderLeft,win->BorderTop,win->GZZWidth,win->GZZHeight,RECTFMT_RGB);

								if (alpha >= 255)
									{
									UBYTE *temp = toarray;
									toarray = fromarray;
									fromarray = temp;
									memset (toarray,0,win->GZZWidth * win->GZZHeight * 3);
									load_image (node->ln_Name,toarray,win->GZZWidth,win->GZZHeight);
									alpha = 0;
									GetSysTime (&starttime);
									node = node->ln_Succ;
									if (!node->ln_Succ)
										node = list->lh_Head;
									}
								}

							if (sigs & winsig)
								{
								struct IntuiMessage *imsg;

								while (imsg = (struct IntuiMessage *) GetMsg (win->UserPort))
									{
									switch (imsg->Class)
										{
									case IDCMP_VANILLAKEY:
										if (imsg->Code == 0x1b)
											cont = FALSE;
										break;
									case IDCMP_CLOSEWINDOW:
										cont = FALSE;
										break;
										}
									ReplyMsg ((struct Message *) imsg);
									}
								}
							}
						while (cont);

						stop_timer (timer);
						}

					if (winarray)  FreeVec (winarray);
					if (toarray)   FreeVec (toarray);
					if (fromarray) FreeVec (fromarray);

					CloseWindow (win);
					}
						
				free_list (list);
				}
			close_timer (timer);
			}
		FreeArgs (rdargs);
		}
	else
		{
		PrintFault (IoErr(),NULL);
		return (RETURN_FAIL);
		}

	return (RETURN_OK);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/
